#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2014 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Traces Splitter Script  -*-                        #
#                                                                              #
#                                                                              #
#  This script splits the traces file produced by tester / instrumented        #
#  application into components-specific traces files.                          #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   $NAME          : name of test node (used as a prefix for traces files)     #
#   $ATxMODE       : use x component                                           #
#                                                                              #
################################################################################
#$Id: Split.pl,v 1.100.4.15 2011/05/04 11:50:48 rfrance Exp $
################################################################################

require $SCRIPTDIR . "rtErrors.pl";
require $SCRIPTDIR . "readLogFile.pl";
# Kinds of output files
$tdf = 1;
$tio = 2;
$rio = 3;
$tpf = 4;
$tqf = 5;
$tgf = 6;
$tzf = 7;
$dcp = 8;
$ccf = 9;
$undefined = 10;

%tag_code = {TDF=>'T',TIO=>'C', RIO=>'R', TPF=>'P', TQF=>'Q', TGF=>'G', TZF=>'Z', DCP=>'D', CCF=>'F'};
@log_codes = ("ERR", "TDF", "TIO", "RIO", "TPF", "TQF", "TGF", "TZF", "DCP", "CCF");
@ext = ("err", "tdf", "tio", "rio", "tpf", "tqf", "tgf", "tzf", "dcp", "ccf");
@used   =   (0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
@opened =   (0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
@condition =(0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

$tdf_file_mode = "w";
$preSplitDone; # set when studio or trtpd or other plit mechanism has been used
# Return the file type matching an opcode
sub GetOpcodeProduct
{
  my $code=shift;
  return $tio if(index(",TB,TP,TE,FT,DC,NT,DT,TA,CO,EI",$code)>0) ;
  return $tpf if($code=~m/^P[FEBOUTSYXG]/) ;
  return $tdf if($code=~m/^M[ERCS]/) ;
  return $tdf if($code=~m/^T[MISUX]/);
  return $tdf if($code=~m/^S[IOAFTY]/);
  return $tdf if(index(",CS,CI,HS,AC,BA,DB,DI,EA,ED,GL,ID,IN,LE,LK,LS,NI,NO,OT,US",$code)>0);
  return $tqf if($code=~m/^Q[CFSUTD]/) ;
  return $tgf if($code=~m/^C[FTDP]/) ;
  return $tzf if($code=~m/^Z[FTDM]/) ;
  return $ccf if($code eq 'CM') ;
  return $dcp if($code eq 'DA') ;
return 0;
}

# Split $traces_file
sub ExecSplit #($$)
{
  my ($traces_file, $print_warnings) = @_;

  my $product = 0;
  my $line = 0;
  my $FTline;
  my $warning_printed = 0;
  my $rejected = "atlout.rej";
  my $text = "";
  my $today = GetDate();
  my $endOfFileFound=0;
  my %SourceFiles;

  if($ENV{ATL_SPLIT_CHECK})
  {
  my $status=0;
  my $tdpEndLine=0;
  my $covState=0;
  my $covCO=0;
  my $lineStatus=0; 
      if(open (TRACES, "<$traces_file"))
      {
	  while(<TRACES>)
	  {
	    $text = $_;
	    chomp($text);
	    $line++;

	    if (/^([A-Z][A-Z])( .*)?/ &&
		($product = GetOpcodeProduct ($1)))
	    {
	      if($product == $tio)
	      {
		  if(/^T[ABPE]/) {
			if($covState!=3) { $lineStatus=2; $covState=3;}
			if($covCO){
			    $lineStatus=1 unless(/^T[ABPE]\s[0-9a-f]+\s[0-9a-f]+$/);
			} else {
			    $lineStatus=1 unless(/^T[ABPE] [0-9a-f]+$/);
			}
		  }elsif(/^FT/) {
			$lineStatus=1 unless(/FT [0-9a-f]{1,8}$/);
			$lineStatus=2 unless($covState & 1); # only 0 and 2
			$covState= 2;
		  }elsif (/^DC/){
			$lineStatus=1 unless(/DC [0-9a-f]{1,8}$/);
			$lineStatus=2 if($covState!=2);
			$covState= 3;
		  }elsif (/^NT/){
		  	$covCO=0;
		  	$lineStatus=1 unless(/^NT$|^NT\s(\"[^\"]*\")((\s[0-9a-fA-F]+)?(\s[0-9a-fA-F]+)?)$/);
			$lineStatus=2 if($covState==2);
		  	$covState=1;
		  }elsif (/^DT/){
		  	$lineStatus=1 unless(/^DT 0$|^DT\s(\"[^\"]*\")$/);
		  }elsif (/^CO/){
		  	$covCO=1;
		  }elsif (/^EI/){
				$status=1;
			print "ERROR: $_ \n";
		  }
	      }
	    }
	    elsif (/^I[0-9] /)
	    {
	    }
	    elsif (/^[A-FGHJ-ORTV-Z][0-9]+.*$/)
	    {
	      if(/^F[12] / && !defined($Ini{RTRT_FLOAT_DUMP})){print "#RTRT_FLOAT_DUMP test line $text at line $line\n";}
	      if(/^D[0-4] -/) {print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1 of %2\n", "",$line,$traces_file);next;}
	    }
	    elsif (/^[A-FGHJ-ORTV-Z]\([0-9A-F]*\}\s.*$/) #{ balance for editor
	    {
			if(/^WA\s*(.)(.*)$/){
				$status=1;
		      		my $info=""; $info="($2)" if($2);
				print "Runtime Error: ",ATL_ERROR($1)," $info\n";
				next;
			}
	    }
	    else
	    {
	      if(/^tdp end/) {$tdpEndLine=1;next;}
	      elsif(/^tdp/) {print; next;} #opcode used by TDP, line suppressed
	      elsif(/^WA\s*(.)(.*)$/){
	      	$status=1;
	      		my $info=""; $info="($2)" if($2);
			print "Runtime Error: ",ATL_ERROR($1)," $info\n";
			next;
	      }
	      $status=1;
	      print &PerlTranslate("Split.pl", "%1:%2: unrecognized line (file %3 contains all the rejected lines)\n", "%1 is the file being split, %2 is the line number and %3 is name of the file to look into", $traces_file, $line, $rejected) if (!$warning_printed);
	      $warning_printed = 1;
	    }
	    if($lineStatus)
	    {
	    	my $errType="";
			$status=1;
			$errType="line format" if($lineStatus ==1);
			$errType="unexpected sequence" if($lineStatus ==2);
	    	print &PerlTranslate("Split.pl", "ERROR: %1 at line %2 of %3. \"%4\"\n","",$errType,$line,$traces_file,$text);
	    	$lineStatus=0;
	    }
	  }
	  close (TRACES);      
	  unless($tdpEndLine)
	  {
	      $status=1;
	      print &PerlTranslate("Split.pl", "ERROR: end of Execution not found in %2\n", "",$traces_file);
	  }
      } else {
	      print &PerlTranslate("Split.pl", "unable to open the file %1\n", "", $traces_file);
	      $status=1;
      }
      if($status){
      	   print &PerlTranslate("Split.pl", "Error found in file %1  => file not split\n", "", $traces_file);
	   return $status;
      }
  }
  open (TRACES, "<$traces_file") || return 5;
  open (REJECTED, ">$rejected") || ($warning_printed = 0);
  $line = 0;
  while(<TRACES>)
  {
    $text = $_;
    $line++;

    if (/^([A-Z][A-Z])( .*)?/ &&
        ($product = GetOpcodeProduct ($1)))
    {
      if(/^(SF|FT|DC|DT|PT|QD|QT|NT|ZD|CD|SG|PG)/){
         if(/^FT/){
            $FTline= $_;
            next;
         }elsif (/^DC\s([0-9a-f]*)/){
            if(($1 == 0) && ($FTline=~m/^FT\s+0\s*$/)) {print &PerlTranslate("Split.pl", "WARNING: wrong coverage source identification. Make sure the static variable initialization is supported on target. Uncheck RTRT_VARIABLE_INIT_SUPPORTED in TDP may fix the issue\n");}
            print { $handles[$product] } $FTline if ($opened[$product]);
         }elsif (/^S([FG])\s+([0-9a-f]+)\s+h([0-9a-f]+)\s+h([0-9a-f]+)\s*$/){ #SF 1 hb22272e7 hacd3a32
            $SourceFiles{$2}="$3 $4";
         }elsif (/^PG\s+([0-9a-f]+)\s+(.+)$/){ #PG 1 .....
            $_ = "PF $SourceFiles{$1} $2\n" if(defined $SourceFiles{$1});
         }elsif (/^([QN]T)\s*$/){
            $_= "$1 \"$ENV{ATL_TEST_NAME}\"\n" if($ENV{ATL_TEST_NAME});
         }elsif (/^PT\s*$/){
            $_= "PT \"$ENV{ATL_TEST_NAME}\" \"$today\"\n" if($ENV{ATL_TEST_NAME});
         }elsif (/^PT \"(.*)\"$/){
            $_= "PT \"$1\" \"$today\"\n";
         }elsif (/^(DT|QD) (0|\"\")$/ || /^(DT|QD)$/){
            $_= "$1 \"$today\"\n";
         }elsif (/^(ZD|CD)$/){
		    my $testFileName='';
			$testFileName= ' '.$ATU_INTERMEDIATE_FILE if($ATUMODE);
            $_= "$1 \"$today\"$testFileName\n";
         }
      }
      $used[$product] = 1;
      print { $handles[$product] } $_ if ($opened[$product]);
    }
    elsif ($ATVMODE &&
           /^I[0-9] /)
    {
      if(/^(I7 [0-9a-f]+ x -)[0-9a-f]/) {$_=$1;print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1\n", "",$line);}
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]);
    }
    elsif ($ATUMODE &&
           /^[A-FGHJ-ORTV-Z][0-9]+.*$/)
    {
      if(/^D[0-4] -/) {print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1\n", "",$line);next;}
      if (/^H0$/){
          $_= "H0 $today\n";
      }
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]);
    }
    elsif ($ATUMODE &&
           /^[A-FGHJ-ORTV-Z]\([0-9A-F]*\}\s/) #{ balance for editor
    {
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]);
    }
    elsif ($ATSMODE &&
           (/^[a-df-z][0-9]+.*$/ ||
            /^f[bt][0-9]+.*$/ ||
            /^i[c-e][0-9]+.*$/ ||
            /^sd[0-9]+.*$/ ||
            /^t[fn][0-9]+.*$/ ||
            /^ATTOL SystemTest .*$/))
    {
      if (/^h 0/) {
          $_= "h $today\n";
      }
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]);
    }
    else
    {
      if(/^tdp\s(.*)$/) {$endOfFileFound=1 if($1 eq "end"); next;} #opcode used by TDP, line suppressed
      if(/^WA\s*(.)(.*)$/){
	my $info=""; $info="($2)" if($2);
	print "Runtime Error: ",ATL_ERROR($1)," $info\n";
      	next;
      }

      print &PerlTranslate("Split.pl", "%1:%2: unrecognized line (file %3 contains all the rejected lines)\n", "%1 is the file being split, %2 is the line number and %3 is name of the file to look into", $traces_file, $line, $rejected) if (!$warning_printed);
      $warning_printed = 1;
      print REJECTED "$line:$text";
    }
  } # while
  close (REJECTED);
  if (!$warning_printed) {
    unlink ($rejected);
  } else {
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
    print  LOG "DUM $rejected\n";
    close( LOG );
  }

  close (TRACES);
  print &PerlTranslate("Split.pl", "Result traces are not complete ($line lines), the application under test has SHUTDOWN DURING ITS EXECUTION\n", "") unless ($endOfFileFound || $ATSMODE);
  return 0;
}

sub OpenDestFile #($$$)
{
   my ($type, $name, $mode) = @_;

   $output_files[$type] = $name;
   $handles[$type] = FileHandle->new ();
   if ($mode eq "a") {
     if(($warningTooBigFile) && ((-s $name) >4500000)){ #4.5MB
        $warningTooBigFile=0;
        print &PerlTranslate("Split.pl", "\nThe result files are growing and might be difficult to show. we suggest to reset the results before the next execution (see the report viewer menu)\n\n", "");
     }
     open ($handles[$type], ">>$name")
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   } else {
     open ($handles[$type], ">$name")
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   }
   $opened[$type] = 1;
}

sub splitlogIt #($$)
{
print  LOG shift,' ',shift,"\n" ;
}

sub SplitTraces #($)
{
   my $tracesfile = shift;

   my $warnings = 0;
   my $filen = 0;
   my $product = 0;
   my $one_file = "";
   my $dirNameDot = $NAME.'.';
   $dirNameDot = "$INTDIR/$NAME." if($INTDIR ne '');

   $condition[$tdf]=($ATOMODE || $ATKMODE || $ATTMODE || $ATJMODE);
   if ($ATOMODE || $ATKMODE || $ATTMODE && !$ATJMODE)
   {
     $filen++;
     $product = $tdf;
     $TDFFILE = $dirNameDot.$ext[$tdf] unless($TDFFILE);
     $one_file = $TDFFILE;
   }
   
   if ($ATCMODE || $DCPMODE || ($ATGMODE && !$ATG_MODULE_AS))
   { #DataCoupling and CtrlCoupling:Function use coverage results
     $condition[$tio]=1;
     $filen++;
     $product = $tio;
     my $exttio=$ext[$tio];
	 $ATC_TIO_NAME=$dirNameDot.$exttio unless($ATC_TIO_NAME);
	 $one_file = $ATC_TIO_NAME;
	 
	 $condition[$dcp]=1 if($DCPMODE);
	 $condition[$ccf]=1 if($ATGMODE && !$ATG_MODULE_AS);
	 
     $DCP_DCP_NAME=$ATC_TIO_NAME; $DCP_DCP_NAME=~s/\Q$exttio\E$/$ext[$dcp]/;
     $CCF_CCF_NAME=$ATC_TIO_NAME; $CCF_CCF_NAME=~s/\Q$exttio\E$/$ext[$ccf]/;
   }
   
   if ($ATUMODE || $ATVMODE || $ATSMODE)
   {
     $condition[$rio]=1;
     $filen++;
     $product = $rio;
     if ($ATUMODE || $ATVMODE)
     {
	$RIOFILE = $dirNameDot.$ext[$rio] unless ($RIOFILE);
     }
     elsif ($ATSMODE)
     {
	$RIOFILE = "$tracesfile.date";
     }
     $one_file = $RIOFILE;
   }
   if ($ATPMODE || $ATAMODE)
   {
     $condition[$tpf]=1;
     $filen++;
     $product = $tpf;
     $ATP_TPF_NAME=$dirNameDot.$ext[$tpf] unless($ATP_TPF_NAME);
	 $one_file = $ATP_TPF_NAME;
   }
   if ($ATQMODE)
   {
     $condition[$tqf]=1;
     $filen++;
     $product = $tqf;
     $ATQ_TQF_NAME=$dirNameDot.$ext[$tqf] unless($ATQ_TQF_NAME);
	 $one_file = $ATQ_TQF_NAME;
   }
   if ($ATGMODE && $ATG_MODULE_AS)
   { #CtrlCoupling:Comilation Unit  uses tgf results
     $condition[$tgf]=1;
     $filen++;
     $product = $tgf;
	 $one_file = $ATG_TGF_NAME;
   }
   #ATG_TGF_NAME must be set in any case for eclipse
   $ATG_TGF_NAME=$dirNameDot.$ext[$tgf] unless($ATG_TGF_NAME);
   if($ATAMODE && $ATZMODE)
   {
     $condition[$tzf]=1;
     $filen++;
     $product = $tzf;
     $ATZ_TZF_NAME=$dirNameDot.$ext[$tzf] unless($ATZ_TZF_NAME);
	 $one_file = $ATZ_TZF_NAME;
   }

   $preSplitDone='';
   #### generate LOG
   if($SOCKET_UPLOAD)
   {
      print &PerlTranslate("Split.pl", "Trtpd Splitting '%1' traces file...\n", "%1 is the name of the file to be split", $tracesfile);
       # assuming trtpd is currently splitting ATT and ATC other lines are written into $tracesfile
      my $waitlong;
      while (!(-f "trtpd.log") && $waitlong < 5){sleep(2);$waitlong +=2;}
	  if(open(INL,"trtpd.log")) {
      if ($LOGFILENAME) {
          open ( LOG, ">>$LOGFILENAME" ) or print &PerlTranslate("Split.pl", "Cannot open log file.\n");
		}
		while(<INL>){
			next if(/^#/);
			if(/^(\w\w\w)\s+(.+)$/){
				print LOG if ($LOGFILENAME);
				$preSplitDone.=$tag_code{$1};
			}
		} close INL;
		close ( LOG ) if ($LOGFILENAME);
	  } else {
			print &PerlTranslate("Split.pl", "Trtpd did not end properly... assuming trace and coverage split only\n");
			$preSplitDone='CT';
      }
   }
   elsif ( !$ATSMODE && !$ATL_MULTI_PROCESS && (($ATTMODE || $ATJMODE) && $ATT_ON_THE_FLY ) || ($ATCMODE && $ATC_ON_THE_FLY))
   {
      print &PerlTranslate("Split.pl", "Studio Splitting '%1' traces file...\n", "%1 is the name of the file to be split", $tracesfile);

      # assuming gui is currently splitting ATT and ATC other lines are written into "unsplitLines.spt"
      #old version $UIriofile=$INTDIR."/atlout.rio"; #all rio line are written in this file
	  $remaining=$INTDIR."/unsplitLines.spt"; #all other lines are written in this file
      sleep (3) ;
      #print "wait End Mark\n";
      my $waitlong=0;
      my $fsize;
      while (!(-f $INTDIR."/atlendtest.txt") && $waitlong < 15){
      	$fsize=(-s $remaining);sleep(2);$waitlong +=2 if($fsize==(-s $remaining));#print " $waitlong.$fsize ";
      }
      unlink($INTDIR."/atlendtest.txt");
      #The split for TPF related information is done by SplitJvmpi for Java.
      if ($LOGFILENAME) {
        open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
        if (-f $TDFFILE) {print  LOG "TDF $TDFFILE\n"; $preSplitDone='T';}
        if (-f $ATC_TIO_NAME) {print  LOG "TIO $ATC_TIO_NAME\n"; $preSplitDone.='C';}
        print  LOG "DUM $tracesfile.full.bak\n";
        close ( LOG );
      }
	  RenameFile($tracesfile,"$tracesfile.full.bak");
	  RenameFile($remaining,$tracesfile); # to be split in later time	 
   }
   elsif (0 && $filen == 1)
   # Optimize when only one feature used disactivated while tools does not parse tdp end properly
   {
     # No use to split file : rename only
     print &PerlTranslate("Split.pl", "Result file looks empty\n") unless(-s $tracesfile);
     RenameFile ($tracesfile, $one_file);
     RegisterGenFiles("$log_codes[$product] $one_file");
	 $used[$product] = 1;
     print &PerlTranslate("Split.pl", "Split done.\n");
     return 0;
   }

   ### now Do it
   print &PerlTranslate("Split.pl", "Splitting '%1' traces file...\n", "%1 is the name of the file to be split", $tracesfile);

   if ($condition[$tdf] && $preSplitDone!~ m/T/) #%tag_code{TDF}
   {
     OpenDestFile ($tdf, $TDFFILE, $tdf_file_mode);
     my ($d,$n,$e) =SplitFileName($tracesfile);
     print { $handles[$tdf] } "NO \"system\" \"$n\"\nCI 0\n" if($tdf_file_mode eq "a");
   }

   if ($condition[$tio] && $preSplitDone!~ m/C/) #%tag_code{TIO}
   {
     OpenDestFile ($tio, $ATC_TIO_NAME, "a");
   }

   if ($condition[$tpf] && $preSplitDone!~ m/P/) #%tag_code{TPF}
   {
     OpenDestFile ($tpf, $ATP_TPF_NAME, "a");
   }

   if ($condition[$tqf]  && $preSplitDone!~ m/Q/) #%tag_code{TQF}
   {
     OpenDestFile ($tqf, $ATQ_TQF_NAME, "a");
   }
   if ($condition[$tgf]  && $preSplitDone!~ m/G/) #%tag_code{TGF}
   {
     OpenDestFile ($tgf, $ATG_TGF_NAME, "a");
   }
   if ($condition[$ccf]  && $preSplitDone!~ m/F/) #%tag_code{CCF}
   {
     OpenDestFile ($ccf, $CCF_CCF_NAME, "w");
   }
   if ($condition[$dcp]  && $preSplitDone!~ m/D/) #%tag_code{DCP}
   {
     OpenDestFile ($dcp, $DCP_DCP_NAME, "w");
   }
   if ($condition[$tzf]  && $preSplitDone!~ m/Z/) #%tag_code{TZF}
   {
     OpenDestFile ($tzf, $ATZ_TZF_NAME, "a");
   }
   
   if ($condition[$rio] && $preSplitDone!~ m/R/) #%tag_code{RIO}
   {
     OpenDestFile ($rio, $RIOFILE, "w");
   }
   my $retSplit=ExecSplit ($tracesfile, $warnings);
   for ($i = $tdf; $i < $undefined; $i++)
   {
     if ($opened[$i])
     {
       close ($handles[$i]);
     }
	 else
     {
	   print &PerlTranslate("Split.pl", "Warning: %1 traces were ignored.\n",$log_codes[$i]) if ($warnings  && $used[$i]);
     }
   }

   if ($retSplit != 0)
   {
     if($retSplit == 5){
		print &PerlTranslate("Split.pl", "Cannot open traces file '%1'. Split aborted.\n\n", "%1 is the name of the file to be split", $tracesfile);
	 } else {
     print &PerlTranslate("Split.pl", "Error while splitting traces file '%1'. Split aborted.\n\n", "%1 is the name of the file to be split", $tracesfile);
     }
     return $retSplit;
   }

   open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n") ;

   if ($condition[$rio])
   {
#     CloseDestFile ($rio);
     splitlogIt($log_codes[$rio],$output_files[$rio]) ;
   }

   if ($condition[$tdf])
   {
#     CloseDestFile ($tdf);
	splitlogIt($ATSMODE?"TDA":$log_codes[$tdf],$output_files[$tdf]);
   }

   if ($condition[$tio])
   {
#     CloseDestFile ($tio);
     splitlogIt($log_codes[$tio], $output_files[$tio]) ;
   }

   if ($condition[$tpf])
     {
#     CloseDestFile ($tpf);
     splitlogIt($ATSMODE?"TDA":$log_codes[$tpf],$output_files[$tpf]);

        my $memsum=MemproSummary($output_files[$tpf]);
        print  LOG "DUM $memsum\n" if($memsum);
     
   }

   if ($condition[$tqf])
   {
#     CloseDestFile ($tqf);
     splitlogIt($ATSMODE?"TDA":$log_codes[$tqf],$output_files[$tqf]);
   }
   
   if ($condition[$tgf])
   {
#     CloseDestFile ($tgf);
     splitlogIt($ATSMODE?"TDA":$log_codes[$tgf],$output_files[$tgf]);
   }
   if ($condition[$ccf])
   {
     splitlogIt($ATSMODE?"TDA":$log_codes[$ccf],$output_files[$ccf]);
   }
   if ($condition[$dcp])
   {
     splitlogIt($ATSMODE?"TDA":$log_codes[$dcp],$output_files[$dcp]);
   }

   if ($condition[$tzf])
   {
#     CloseDestFile ($tzf);
     splitlogIt($ATSMODE?"TDA":$log_codes[$tzf],$output_files[$tzf]);
   }
   if($ENV{'ATTOLSTUDIO_VERBOSE'})
   {
      unlink ("$tracesfile.bak");
      rename ($tracesfile,"$tracesfile.bak");
      print  LOG "DUM $tracesfile.bak\n" if($STUDIO_IDE);
   }
   else
   {
      unlink ($tracesfile);
   }
   close( LOG ) ;#unless($ATSMODE);
   FloatConvert($RIOFILE) if(($used[$rio]) && defined($Ini{RTRT_FLOAT_DUMP}));
   $used[$tio]=1 if($preSplitDone=~ m/C/);
   $used[$tdf]=1 if($preSplitDone=~ m/T/);
   my $what;
   $what.='Cov ' if ($used[$tio]);
   $what.='Mem ' if ($used[$tpf]);
   $what.='Perf ' if ($used[$tqf]);
   $what.='Ccoup ' if ($used[$tgf]);
   $what.='CCoup ' if ($used[$ccf]);
   $what.='Dcoup ' if ($used[$dcp]);
   $what.='StackZ ' if ($used[$tzf]);
   $what.='Trace ' if ($used[$tdf]);
   print &PerlTranslate("Split.pl", "Traces file successfully split. $what\n");
   return 0;
}

sub MemproSummary
{
   my $file=shift;
   my $out=$file; $out=~s/\.tpf$//;
   my %ERR;
   my $x;
   open IN, "$file" || return 0;
   while(<IN>){
   	if(/^PE (\w+)/){
   	   my $e=$1;
   	   unless($e=~ m/CHK|FIU|STKS/){
   	      $ERR{$e}++ ;
   	      $x++;
   	   }
   	   #print "Mempro $e, $_, $ERR{$e}\n";
   	}
   }
   close IN;
   if($x) {
      open OUT, ">$out.atp.txt";
      print OUT "---   Memory errors Summary\n";
      foreach (sort keys %ERR){
   	print OUT "$_ found= $ERR{$_} times\n";
      }
      print OUT "Total Memory errors found=$x\n";
      close OUT;
      print "$x Memory Errors found\n";
      return "$out.atp.txt";
   }
   return 0;
}

# splitAfterExec is used only after instrumented application execution
sub splitAfterExec
{
	if ( $ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || ($ATAMODE && $ATZMODE) || $ATGMODE || $DCPMODE || $ATKMODE )
	{
	  # when socket uploader is used SplitTraces checks created files and make the log
          #in Multi-process the spt file name has changed as xxxxx.pid.spt instead of xxxxx.spt
	       my $exeInstr=0;
		  if($ATL_MULTI_PROCESS){
			 my ($dir,$name,$ext) = SplitFileName($ATL_OUT_FILE);
	          	 sleep(2);#wait for all spt file closed
			 opendir(DIR,"$dir") || die &PerlTranslate("trtsplit.pl", "Error: unable to open directory %1.\n","%1 directory name",$dir);
			 while($f=readdir(DIR)) {
				if($f=~ /$name\..*\.$ext$/){
				SplitTraces ($dir.$f);
				$tdf_file_mode="a";
				}
			 }
			 closedir(DIR);
		  } else {
			SplitTraces ($ATL_OUT_FILE); # LOG are made inside SplitTraces
		  }
		  #---------------------------------------------------------------------
		  ## Coverage check ...
		  if ( $ATCMODE )
		  {
		      if ( ! $used[$tio] )
		      {
			  print &PerlTranslate("trtsplit.pl", "No Coverage results found.\n");
			  $exeInstr++;
		      }
			  #in studio CIO file is generated. Not in eclipse
			  if($TestRTCC)
			  {
			    unless($NO_JUSTIFY) {
				  # get the FDC file in the LOG to be used by the genJustifyFDC
				  readLogFile('s') unless(@FDCFILES);
			      $ATC_JUSTIFY= genJustifyFDC(\@FDCFILES);
				  RegisterGenFiles("TIO $ATC_JUSTIFY") if($ATC_JUSTIFY ne '');
				  #print ">>>>>>>## TIO $ATC_JUSTIFY:",@FDCFILES,"\n";
				}
			  } else {
				  atc_postpro($ATC_TIO_NAME,""); # no CIO in this node
			  }
		  }

		  #---------------------------------------------------------------------
		  ## Manage TDF Traces ...
		  if ( $ATTMODE )
		  { # No attsplit nor attloop on the fly
		      if ( $used[$tdf] )
		      {
			      # print $TDFFILE, "size=",(-s $TDFFILE),"\n";
			      if((-s $TDFFILE) > ($ATT_SPLIT_SIZE*1000)) {
				  # get the TSF file in the LOG to be used by the PostAttManage
				  readLogFile('s') unless(@TSFFILES);

				  #then split the big file
				  PostAttManage ($TDFFILE) if(@TSFFILES);
			      }
		      }
		      else
		      {
			  print &PerlTranslate("trtsplit.pl", "No Tracing results found.\n");
			  $exeInstr++;
		      }
		  } #ATTMODE==1

		  #---------------------------------------------------------------------
		  ## Memory Profiling check ...
		  if ( $ATPMODE || $ATAMODE)
		  {
		      if ( ! $used[$tpf] )
		      {
			  print &PerlTranslate("trtsplit.pl", "No Mempro results found.\n");
		      }
		  } #ATPMODE==1

		  #---------------------------------------------------------------------------
		  ## Performance Profiling check ...
		  if ( $ATQMODE )
		  {
		      if ( ! $used[$tqf] )
		      {
			  print &PerlTranslate("trtsplit.pl", "No Perfpro results found.\n");
			  $exeInstr++;
		      }
		  } #ATQMODE==1

	  #---------------------------------------------------------------------------
	  ## CtrlCoupling check ...
	  if ( $ATGMODE )
	  {
	      unless ($ATG_MODULE_AS ? $used[$tgf] : $used[$tio])
	      {
		  print &PerlTranslate("TestRTexec.pl", "No Control Coupling results found.\n");
		  $exeInstr++;
	      }
	  }
	  #---------------------------------------------------------------------------
	  ## DataCoupling check ...
	  if ( $DCPMODE )
	  {
	      if ( ! $used[$tio] )
	      {
		  print &PerlTranslate("TestRTexec.pl", "No Data Coupling results found.\n");
		  $exeInstr++;
	      }
	  }
	  #---------------------------------------------------------------------------
	  ## Performance Profiling check ...
	  if ($ATAMODE && $ATZMODE)
	  {
	      if ( ! $used[$tzf] )
	      {
		  print &PerlTranslate("TestRTexec.pl", "No stack results found.\n");
		  $exeInstr++;
	      }
	  }	  
		  #---------------------------------------------------------------------------
		if($exeInstr == 5) {
		  print "No results. Clean the project to instrument the application.\n";
		  return 1;
		}
	    } #ATCMODE==1 || ATTMODE==1 || ATPMODE==1 || ATQMODE==1 || $ATKMODE==1

return 0;
}

sub ParseAtsRioFile #($)
{
   my $riofile = shift;
   SplitTraces ($riofile) and return 1;
   rename ("$riofile.date", $riofile) or return 1;
   return 0;
}

sub FloatConvert #($RIOFILE)
{
    my $file=shift;
    my $exe;
    my $PATHSEP='/';
    if($^O=~/win32/i) {
    	$exe='.exe' ;
    	$PATHSEP='\\';
    }
    if(-f $file){
	if(-f "$ENV{ATLTGT}/cmd/extrdump$exe") {
		$exe=$ENV{ATLTGT}.$PATHSEP.'cmd'.$PATHSEP."extrdump".$exe
	} else {
		$exe="extrdumpieee".$exe;
	}
	SystemP("$exe $file $file.tmp");
	rename("$file.tmp",$file);
    }
}

# Decodes given execution log file and generates SPT trace file
sub splitMarkerFilter #($$$)
{
my $log=shift;
my $out=shift;
my $mark=shift;
$mark='^' unless($mark);
    if(open (IN, $log)) {
	open (OUT, ">$out") || die "Unable to create '$out': $!\n";
	my $dumpBuffer = 0;
	while(<IN>) {
	    foreach my $chunk (split (/\Q$mark\E/)) {
		print OUT $chunk if($dumpBuffer); # SPT trace
		print $chunk unless($dumpBuffer); # stdout trace
		$dumpBuffer = !$dumpBuffer;
	    }
	    $dumpBuffer = !$dumpBuffer;
	}
	close (IN); close (OUT);
    } else {print "Unable to open '$log': $!\n";}
}

# Decode given execution log file from debugger hexa dump and generates SPT trace file
# the replace param should provide each char in hex format separated by a space. The number of char per line does not matter.
# startString param is optional defaulting to gdb dump format.
# Do not forget that replace param must have simple and double cot as follow>
# hexaDumpFilter('RTRT.log',$out,'^0x[0-9A-Fa-f]+\s+<atl_buffer.+\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..).*$','"$4 $3 $2 $1 $8 $7 $6 $5 $12 $11 $10 $9 $16 $15 $14 $13"'); #for le
# hexaDumpFilter('RTRT.log',$out,'^0x[0-9A-Fa-f]+\s+<atl_buffer.+\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..)\s+0x(..)(..)(..)(..).*$','"$1 $2 $3 $4 $5 $6 $7 $8 $9 $10 $11 $12 $13 $14 $15 $16"'); #for be
sub hexaDumpFilter #($$$$$)
{
my $in=shift;
my $out=shift;
my $find=shift;
my $replace=shift;
my $startString=shift;
$startString='<atl_buffer>:' unless($startString); #default gdb dump
# remove bad char in file.
    if(open (IN, $in)) {
	open (OUT, ">$out") || die "Can't write '$out': $!\n";
	my $On=0;
	$On=1 if($startString eq '');
	while (<IN>)
	{
 		$On=1 if($startString ne '' && /$startString/);
	 	$_=~ s/0xa/0x0a/;
 		next unless(s/$find/$replace/ee);
	 	foreach(split(/\s+/))
	 	{
 	 		$On=0 if(/00/);
 			print OUT chr hex($_) if($On);
# 	 		print    chr hex($_) if($On);
 		}
	}
	close (IN); close (OUT);
    } else { print "Can't open '$in': $!\n";}
}

1;
