#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2008 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
# This file expects the name of a parameter file its first and only argument
##
# It will return
# 0 The test executed correctly
# 1 The user interface could not be run
# 5 The project could not be built
#################################################################################

use warnings;
use strict;
## for debug
$main::SCRIPTDIR=$ENV{TESTRTDIR}."/lib/scripts/";
require $main::SCRIPTDIR . "Fcts.pl";
# Set to one to enable debugging
my $DEBUG = 0;
if(defined($ENV{TESTRTRQMLOG})) {
	$DEBUG =1;
	open (DEBUGLOG,">>$ENV{TESTRTRQMLOG}") || die &PerlTranslate("TestRTRqm.pl", "Error: Unable write on log file %1\n", "%1 is the log file",$ENV{TESTRTRQMLOG});
}
# The script expects one parameter which is the name of the parameter file
my $logmsg="$0 needs one, and only one parameter which is the name of the file containing the parameters to run the test\n" ;
$DEBUG && print DEBUGLOG $0,' ', join(' ',@ARGV),"\n";
die $logmsg unless (@ARGV == 1);
my $paramFile = $ARGV[0];
my $Drive;
my $resultsFile;
my $htmlReport;
my $BuildlogFile;
my $studioMode=0;
my $errMount="";

$DEBUG && print DEBUGLOG GetDate(), " Read of $paramFile:\n";

# Does the parameter file exist and is it readable?
open (IN, $paramFile) || die "Cannot open the file $paramFile for reading:\n$!\n";

my %params;
my $lineno = 0;
while (<IN>) {
  $DEBUG && print DEBUGLOG ;
  $lineno++;
  next if (/^\s*$/); # Ignore empty lines
  next if (/^\s*#/); # Ignore comments
  chomp; # Remove newline characters
  # now must have a line with the format key=value
  if (m,\s*(\w+)=(.+)\s*$,) {
    my $key = $1;
    my $value = $2;
    # Store the key and value in the parameter lookup table
    $params{$key} = $value;
  } else {
    $logmsg="Unrecognized line in the parameter file\n$lineno: $_\n";
    $DEBUG && print DEBUGLOG $logmsg;
    die $logmsg;
  }
}
close (IN);
# key used by the RQM adapter:
#testRTScriptPath is the project file .rtp (args[0])
#testNodeName is the node name to rebuild (args[1]) (filter)
#tdpName is the configuration Name (args[2])
#sharedPrefix is the part of the path that can be shared between server and TestRT install (args[3])
#relativePath is not used here (args[4])
#tempDirName is the temporary directory used to generate html report
#resultSummaryFile is the xml file giving the summary of the executed tests

# Check that we have received the minimum expected keys
#die "Missing the name of the test node to execute. Expected parameter name is \"testNodeName\".\n" unless (exists $params{testNodeName});
$logmsg="Missing the name of the project file containing the tests. Expected parameter name is \"testRTScriptPath\".\n" ;

unless (exists $params{testRTScriptPath}) {
	$DEBUG && print DEBUGLOG $logmsg;
	die $logmsg ;
}

# Remove double backslashes from the name of the project
if($params{sharedPrefix}) {
	$params{sharedPrefix} =~ s/\\\\/\\/g;
	$params{sharedPrefix} =~ s/\\:/:/g;
	if(($params{sharedPrefix} =~ m/^\\\\/) && ($^O =~ /Win32/)) {
		my $mount="net use * ".$params{sharedPrefix};
		$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$mount\n";
		my $out = `$mount` ;
		my $status = $? ;
		my $ret = $status / 256;
		if(($out=~ m/\s(\w\:)\s/) && ($ret ==0)){
			$Drive=$1;
		} else {
			$errMount = $out." error code ".$ret;
		}
	}
}

$params{testRTScriptPath} =~ s/\\\\/\\/g;
$params{testRTScriptPath} =~ s/\\:/:/g;
$params{testRTScriptPath} =~ s/\s+$//;
$params{testRTScriptPath} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);

if($params{tempDirName}) {
	$params{tempDirName} =~ s/\\\\/\\/g;
	$params{tempDirName} =~ s/\\:/:/g;
	$params{tempDirName} =~ s/\s+$//;
	$params{tempDirName} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);
}

# Calculate the log file name
my $logfile =$params{tempDirName}.'/TestRTRqm.log';
my ($testRTScriptDir,$testRTScriptBase,$tmpExt)=SplitFileName($params{testRTScriptPath});
my $logfilecopy =$testRTScriptDir.'TestRTRqm.log';

$DEBUG && print DEBUGLOG "\n", GetDate(), " logfile = $logfile\n";
open (LOG, ">$logfile") || die "Cannot create the log file $logfile\n";
$logmsg="# set environment variable TESTRTRQMLOG to a logFileName.txt to get more details<br>\n";
$logmsg="# environment variable TESTRTRQMLOG is set to a $ENV{TESTRTRQMLOG} to get more details<br>\n" if($DEBUG);
print LOG $logmsg;
print LOG $errMount;
# Now do what you want!

# Then run the test via studio or Visual Test
my $command;

if($params{testRTScriptPath}=~m/\.rtp$/){
	$studioMode=1;
	if($params{testNodeName}) {
		$command = "studio -r \"".$params{testNodeName}."\" \"".$params{testRTScriptPath}."\"";
	} else {
		$command = "studio -r  \"".$params{testRTScriptPath}."\"";
	}
	$command .= " -config \"".$params{tdpName}."\"" if (exists $params{tdpName});
	$command .=  " -html \"".$params{tempDirName}."/htmlDir\"" if (-d $params{tempDirName});
	$htmlReport=$params{tempDirName}.'/htmlDir' if (-d $params{tempDirName});
	#get global summary into 1 result file for futur Adapter release
	#for now the name of this global summary must not be named xxx.summary.txt otherwise it will be read as node result in addition to all other summary
	#globalize the test result in 1 file
	$resultsFile = $params{testRTScriptPath};
	$resultsFile =~s/([\\\/])[^\\\/]*$/$1RQM.results.txt/i;
	$ENV{RTRT_TEST_SUM}=$resultsFile;
	unlink($resultsFile);
	$BuildlogFile=$params{testRTScriptPath};$BuildlogFile=~s,rtp$,log,;
	$DEBUG && print DEBUGLOG "Build log File=$BuildlogFile\n";
	unlink($BuildlogFile);
} elsif ($params{testRTScriptPath}=~m/\.test_suite$/){
	my $TESTSUITE=$params{testRTScriptPath};
	my $WORKSPACE;
	my @dirs=split('[\\\/]',$TESTSUITE);
	$WORKSPACE=shift(@dirs);
	while(!(-d $WORKSPACE.'/.metadata') && @dirs){
		$WORKSPACE.='/'.shift(@dirs);
	}
	$logmsg="WorkSpace Not found in $TESTSUITE\n";
	$DEBUG && print DEBUGLOG $logmsg unless(@dirs);
	die $logmsg unless(@dirs);
	$params{resultFile}="main.test_report" unless(defined($params{resultFile}));
	$WORKSPACE='"'.$WORKSPACE.'"' if($WORKSPACE=~/\s/);
	$TESTSUITE ='/'.join('/',@dirs);# make it relative to workspace
	$TESTSUITE='"'.$TESTSUITE.'"' if($TESTSUITE=~/\s/);
	$command .='"'.$ENV{TESTRTDIR}.'/eclipse" -nosplash -application com.ibm.rational.testrt.test.campaign.launcher.TestLaunchers ';
	$command .='-data '.$WORKSPACE.' ';
	$command .=$TESTSUITE.' ';
	$command .="-xmlfile=\"$params{tempDirName}/$params{resultFile}\" ";
	$command .="-XMLREPORTS ";
	#-BINDIR="<R�pertoire d'install des binaires TestRT>"
	#-TDPDIR="<Liste de r�pertoire o� se trouve les TDPs>"
} else {
	$logmsg="Unknown project file or test_suite. The extension does not match the Expected one in \"testRTScriptPath\".\n" ;
	$DEBUG && print DEBUGLOG $logmsg;
	die $logmsg;
}
$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$command\n";
print LOG GetDate(), " Running the following command:<br>\n$command<br>\n";
$! = 0;
my $status = system ($command);
my $ret = $status / 256;
$DEBUG && print DEBUGLOG GetDate(), " Raw return status $status => $ret\n";
print LOG GetDate(), " Raw return status $status => $ret\n";
# Check the return value and return one of the following values from the script
if($studioMode){
	#TT_FAILURE                0  some of tests executed with errors
	#TT_SUCCESS                1  tests executed correctly without any errors
	#TT_DID_NOT_COMPLETE       5  The project could not be built by studio
	if ($ret == 1) {
	  # There is an ambiguity with the value 1. 1 means an error occurred in system
	  # or the TestRT test was built successfully. $! will tell us which it is
	  if ($! eq "") {
		# TestRT test was built successfully
		$ret = 0;
	  } else {
		# an error occured in system call
		print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret\n$!<br>\n<br>\n";
		$ret =1;
	  }
	} elsif ($ret == 5) {
	  # Build failed
	  print LOG "TestRTRqm.pl: Build of the test failed with error code 5<br>\n";
	} else {
	  print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret<br>\n$!<br>\n" unless ($ret == 0);
	  $ret = 1;
	}
}
else {
	if ($ret != 0) {
		print LOG "TestRTRqm.pl: Build of the test failed with error code $ret<br>\n";
	}
}

if($studioMode){
  if(-f $resultsFile) {
    if(open(RES,$resultsFile)) {
        open(XML, ">$params{tempDirName}/$params{resultSummaryFile}") or die "TestRTRqm.pl: unable to write the result summary file $params{resultSummaryFile}\n";
        print XML "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<RUNS>\n";
        print XML "<LOG>$BuildlogFile</LOG>\n" if(-f $BuildlogFile);
        my @COVFILES;
        my ($TestNodeName,$TestPass,$TestFail,$TestInco,$TestTotal,$TestPath,$TestReport,$branch,$cov);
        $TestInco=0;
	$branch=0;
	$cov=0;
	while(<RES>){
	    if(/^Test node=(.*)$/){$TestNodeName=$1;}
	    if(/^Report file=(.*)$/){$TestReport=$1;$TestPath=$1;$TestPath=~s/[\\\/][^\\\/]\.xrd$//;}
	    if(/^Passed=(.*)$/) {$TestPass=$1;}
	    if(/^Failed=(.*)$/) {$TestFail=$1;}
	    if(/^Inconclusive=(.*)$/) {$TestInco=$1;}
	    if(/^(FDCFILES|TIOFILES)=(.*)$/) {push(@COVFILES,$2);}
	    if(/^Total=(.*)$/) {
		$ret=0;#re-considere return code
	        $TestTotal=$1;
	        my $status="OK";
	        $status="INCONCLUSIVE" if($TestInco);
	        $status="KO" if($TestFail);
#<RUN NAME="th_add" PATH="/prj/th_add.test_harness" DATE="Thu Sep 22 15:25:12 CEST 2011" PASSED="0" INCONCLUSIVE="0" FAILED="1" STATUS="KO" >
#	<REPORT>C:/temp/runtime-New_configuration33/prj/TestRT_win32/th_add.1316697912045.test_report</REPORT>
#</RUN>
	        print XML "<RUN NAME=\"$TestNodeName\" PATH=\"$TestPath\" DATE=\"",GetDate();
	        print XML "\" PASSED=\"$TestPass\" FAILED=\"$TestFail\" INCONCLUSIVE=\"$TestInco\" TOTAL=\"$TestTotal\" STATUS=\"$status\" >\n";
	        print XML "   <REPORT>$TestReport</REPORT>\n";
	        print XML "</RUN>\n";
	        $TestNodeName='';
	        $TestPath='';
	        $TestReport='';
	        $TestPass=0;
	        $TestFail=0;
	        $TestInco=0;
	        $TestTotal=0;
	    }
	}
	if(@COVFILES){
	   if(open(OPT,">attolcov.opt")){
	   	print OPT "@COVFILES\n";
	   	close OPT;
		$command="attolcov  -su=c -output=coverage_results.txt \@attolcov.opt";
		$DEBUG && print DEBUGLOG GetDate(), " Running the following command:\n$command\nattolcov.opt is:@COVFILES\n";
		$! = 0;
		my $status = system ($command);
		my $ret = $status / 256;
		$DEBUG && print DEBUGLOG GetDate(), " Raw return status $status => $ret\n";
		if(open(RCOV,"coverage_results.txt")){
		   while(<RCOV>){
		   	next if(/Multiple cond/);
			if(/\((\d+)\/(\d+)\)/){
			   $cov+=$1; $branch+=$2;
			}
		   }
		   close RCOV;
		   unlink("coverage_results.txt","attolcov.opt") unless($DEBUG);
		}
	   }
	}
	print XML "<COVERAGE_RESULTS COVERED_BRANCH=\"$cov\" BRANCH_NUMBER=\"$branch\" ></COVERAGE_RESULTS>\n" if($branch);
        print XML "</RUNS>\n";
        close XML;
	close RES;
	unlink($resultsFile) unless($DEBUG);
    } else {
        print LOG "TestRTRqm.pl: Unable to read the results file $resultsFile<br>\n";
    }
  } else {
	$DEBUG && print DEBUGLOG $resultsFile," results file not found !!!\n";
  }
  print "<br>The build log is:<br>\n";
  if(open(BUILD,$BuildlogFile)){
	print $_,"<br>" while(<BUILD>);
  }
  close BUILD;
} else {
#Visual Test Mode
  copyEclipseTestSuiteReports("$params{tempDirName}/$params{resultFile}",$params{tempDirName},"$params{tempDirName}/$params{resultSummaryFile}");
}
close (LOG);
copy($logfile,$logfilecopy);
#unlink($logfile) unless(-s $logfile);
system("net use $Drive /delete") if($Drive);
$DEBUG && print DEBUGLOG GetDate(), " exit code = $ret\n";
close DEBUGLOG;
exit($ret);

my $workspace_path;
sub copyEclipseTestSuiteReports
{
  my $resultSummaryFile=shift;
  my $destDir=shift;
  my $destResFile=shift;
  my $xslToBeCopied=1;
  my $translate=0;
  my $covTotal=0;
  my $covCovered=0;
  $DEBUG && print DEBUGLOG "reading summary file $resultSummaryFile\n";
  if(open(XML, "$resultSummaryFile") ) {
	 $DEBUG && print DEBUGLOG "generating results file $destResFile\n";
     if($destResFile ne '') {
	     if(open(OUT,">$destResFile")) {$translate=1;} else { print LOG "Unable to write file $destResFile\n";}
     }
     while(<XML>){
	 if(/<RUN /){
		 my $logf;
		 if(/WORKSPACE_PATH=\"([^\"]*)\"/) {$workspace_path=$1;}
		 if(/LOG=\"([^\"]*)\"/) { $logf=$1;copyReport("/$1",$destDir); }
#		 $translate && print OUT "<RUNS>\n<LOG>$destDir/$logf</LOG>\n";
#		 next;
	 }
	 elsif(/<(TESTSUITECALL|QUANTIFY|PURIFY|TRACE|METRICS|CODEREVIEW|CTRLCOUPLING|DATACOUPLING|WSS|METRICS)_RESULT\s/)
	 {  
	    my $key=$1;
		if(/REPORT_PATH=\"([^\"]*)\"/){
			 copyReport($1,$destDir);
			 $DEBUG && print DEBUGLOG "copy $key report $1\n";
		}
	 }
	 elsif(/<COVERAGE_RESULT\s/)
	 {
		 if(/REPORT_PATH=\"([^\"]*)\"/){
			 copyReport($1,$destDir);
			 $DEBUG && print DEBUGLOG "copy COVERAGE report $1\n";
		 }
		 my $t=0;my $c=0;
		 if(/BRANCH_NUMBER=\"(\d+)\"/){
			 $t=$1;
			 if(/COVERED_BRANCH=\"(\d+)\"/){
				 $c=$1;
				 if($covTotal<$t){
					 $covTotal=$t;
					 $covCovered=$c;
				 } elsif($covTotal==$t && $covCovered<$c){
					 $covCovered=$c;
				 }
			 }
		 }
		 next;
	 }
	 elsif(/<\/COVERAGE_RESULT/)
	 {
		 next;
	 }
	 elsif(/<\/RUN>/)
	 {
		 $translate && print OUT "<COVERAGE_RESULTS COVERED_BRANCH=\"$covCovered\" BRANCH_NUMBER=\"$covTotal\" ></COVERAGE_RESULTS>\n";
	 }
	 #old report format kept for compatibility (before 8011)
         if(/<(REPORT|LOG)>(.+)<\/(REPORT|LOG)>/){
         	my ($d,$b,$e)=SplitFileName($2);
         	#copy($2,$destDir.'/'.$b.'.'.$e);

		# Copy reports
		my @REPORTSEXT = ("test_report","coverage_report","mempro_report","perfpro_report","tgf.html","dcp.html","tzf.html");
		foreach (@REPORTSEXT) {
			my $report_src = $d.'/'.$b.'.'.$_;
			my $report_dest = $destDir.'/'.$b.'.'.$_;

			if (-f $report_src) {
				copy($report_src, $report_dest);
				$DEBUG && print DEBUGLOG "copy report $report_src\n";
				unless(-f $report_dest){
					$DEBUG && print DEBUGLOG "$report_src not found !!!\n" ;
					print LOG "$report_src not found !!!\n";
				}
			}
		}

		# Copy style sheets if not already done
		my @STYLESHEETS = ("testsuite_report","coverage_report","mempro_report","perfpro_report");
		foreach (@STYLESHEETS) {
			my $styleSheet_src = $d.'/'.$_.'.xsl';
			my $styleSheet_dest = $destDir.'/'.$_.'.xsl';

			if (-f $styleSheet_src) {
				copy($styleSheet_src, $styleSheet_dest);
				$DEBUG && print DEBUGLOG "copy styleSheet $styleSheet_src\n";
				print LOG "$styleSheet_src not found !!!\n";
				unless(-f $styleSheet_dest){
				   $DEBUG && print DEBUGLOG "$styleSheet_src not found !!!\n";
				   print LOG "$styleSheet_src not found !!!\n" ;
				}
			}
		}
         }

	 if($translate){
#		 s,<TESTSUITECALL_RESULT ,<RUN ,;
#		 s/\/RUN/\/RUNS/;
#		 s/\/TESTSUITECALL_RESULT/\/RUN/;
        	 print OUT ;
	 }
     }
     close XML;
     close OUT if($translate);
  } else {
     print LOG "TestRTRqm.pl: Unable to read the results file $resultSummaryFile\n";
  }
}
my %stylesheetcopied;
sub copyReport
{
	my ($d,$b,$e)=SplitFileName(shift);
	my $destDir=shift;
	$d=$workspace_path.$d;
	my $report_src = $d.$b.'.'.$e;
	my $report_dest = $destDir.'/'.$b.'.'.$e;
	copy($report_src,$destDir);
	print LOG "$report_src not found !!!\n" unless(-f $report_dest);
	unless($stylesheetcopied{$e}){
		$stylesheetcopied{$e}=1;
		if($e=~m/_report$/){
			$e=~s,test_report,testsuite_report,;
			copy($d.$e.'.xsl',$destDir);
			unless(-f $destDir.'/'.$e.'.xsl') {
			    print LOG "$e.xsl not found !!!\n" ;
				$DEBUG && print DEBUGLOG "$e.xsl not found !!!\n" ;
			}
		}
	}
}
1;
