#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Unit Testing node -*-                            #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   @OBJECTS     : object files                                                #
#   $LDFLAGS     : link flags                                                  #
#   $LIBRARIES   : libraries list for linking                                  #
#   $TESTPROGRAM : executable name                                             #
#   $ARGUMENTS   : executable arguments                                        #
#   $EXECMODE    : execution mode ("NORMAL" or "DEBUG")                        #
#   $AMLFILE     : generated AML report                                        #
#   @OTSFILES    : list of OTS files                                           #
#   $BINDIR      : ATTOL binaries directory                                    #
#   $SCRIPTDIR   : Studio scripts directory                                    #
#   $TARGETDIR   : target package directory                                    #
#   $OUTDIR      : output directory                                            #
#   $LOGFILENAME : generated files log                                         #
################################################################################




# To do output in chronological order when stderr and stdout are both used
use FileHandle;
use File::Copy ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0] unless($main::TestRTCC);

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";
require $SCRIPTDIR . "instrPostpro.pl";

# Target package related functions
require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);
require $SCRIPTDIR . "CmsInfo.pl";

# Target-dependant functions
require $TARGETDIR ."/cmd/atl_func.pl";
require $TARGETDIR ."/cmd/atl_link.pl";
require $TARGETDIR ."/cmd/atl_exec.pl";


################################################################################
# Attol Unit Testing postprocessor execution
# Arguments:
#    $atu_out    : traces file
#    $atu_xrd    : generated AML report
# Return value: attolpostpro return value
# Required global variables:
#    $BINDIR
# Used global variables:
################################################################################
sub atu_postpro #($$$)
{
  my ($atu_out,$atu_xrd,$atu_cio) = @_;

  # Suppress old features
  delete $ENV{'ATTOLLAN'} ;
  delete $ENV{'ATTOLCINC'} ;
  delete $ENV{'ATTOLTMPK'} ;
  delete $ENV{'ATTOLCPPINC'} ;
  delete $ENV{'ATW_NO_USR_INCLUDE'} ;

  # Let's environment
  $ENV{'ATTOLHOME'} = $ATTOLHOME;
  $ENV{'ATUDIR'} = $ATTOLHOME . $PATH_SEPARATOR."lib";
  $ENV{'ATUTGT'} = "$TARGETDIR";

  # Defaults options
  my $atu_postpro_name = "attolpostpro";
  my $atu_options = " ";
  my $ada_mode = 0;

  if ($TDP_LANGUAGE=~/ADA/)
  {
    $ada_mode = 1;
  }
  if ($LOGFILENAME)
  {
    $atu_options .= " -studio_log=\"$LOGFILENAME\"";
  }

  # Update options

  if (($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP") || ($TDP_LANGUAGE eq "ADA83_V34"))
  {
     $atu_postpro_name = "attolpostpro";
  }
  if ($TDP_LANGUAGE=~/ADA/)
  {
     $atu_postpro_name = "attolpostproada";
  }

  if ($ATCMODE && !$ATC_REPORT_FAILED && !$NO_COV_REPORT)
  {
    $atu_options .= " -cio=\"$atu_cio\"";
  }

  my ($d,$ATU_RI2,$e) = SplitFileName($UNITTESTING_TESTSCRIPTNAME);$ATU_RI2="$PROJECT_PATH$PATH_SEPARATOR$ATU_RI2.ri2";
  #my $ATU_RI2 = "$PROJECT_PATH"."$PATH_SEPARATOR"."$NAME.ri2";
  if (($ATU_COMPARE) || ($ATU_REPORT_OPTIONS=~ /-compare/))
  {
    if (-f "$ATU_RI2")
    {
       ($dev1,$ino1,$mode1,$nlink1,$uid1,$gid1,$rdev1,$size1, $atime1,$mtime1,$ctime1,$blksize1,$blocks1) = stat($ATU_RI2);
       ($dev2,$ino2,$mode2,$nlink2,$uid2,$gid2,$rdev2,$size2, $atime2,$mtime2,$ctime2,$blksize2,$blocks2) = stat($UNITTESTING_TESTSCRIPTNAME);
       if ($mtime1 lt $mtime2)
       {
	       die &PerlTranslate("UnitTesting.pl", "Error: Unable to diff with file %1 (test script has been modified)\n       Please re-build without having the 'Diff of last two test runs' option turned on.\n", "%1 is the name of the file", "$ATU_RI2");
       }
       atl_copy("$ATU_RI2", "$atu_out.ri2", 1);
       $atu_options .= " -compare=strict" if ($ATU_COMPARE) ;
    }
    else
    {
       die &PerlTranslate("UnitTesting.pl", "Error: Unable to diff with file %1 (not found)\n       Please re-build without having the 'Diff of last two test runs' option turned on.\n", "%1 is the name of the file", "$ATU_RI2");
    }
  }
  else
  {
    atl_copy("$atu_out.rio", $ATU_RI2, 1);
    RegisterGenFiles("RI2 $ATU_RI2");
  }

  if ($ATU_FILTER == 1)
  {
      $atu_options .= " -RA=ERR";
  }
  elsif ($ATU_FILTER == 2)
  {
      $atu_options .= " -RA=TEST";
  }
  elsif ($ATU_FILTER == 3)
  {
      $atu_options .= " -RA=COLLAPSE";
  }

  if ($ATU_STRATEGY == 0)
  {
      $atu_options .= " -VA=EVAL";
  }
  elsif ($ATU_STRATEGY == 1)
  {
      $atu_options .= " -VA=NOEVAL";
  }
  elsif ($ATU_STRATEGY == 2)
  {
      $atu_options .= " -VA=COMBINED";
  }

  # Postpro execution
  unlink ( "$atu_out.rod" );

  my $atu_command = $BINPATH.$atu_postpro_name . " \"$atu_out\" \"$atu_out.rod\" $atu_options" ." ".$ATU_REPORT_OPTIONS;
  my $postpro_status = SystemP($atu_command);

  if ( ! -f "$atu_out.rod" )
  {
      die &PerlTranslate("UnitTesting.pl", "No result generated.\n");
  }

  if($TPM_GEN){
     # TPM initialization
     $ENV{'ATTOL_TPM_USER'}=$USERNAME;
     $ENV{'ATTOL_TPM_ROOT'}=$ATTOL_TPM_ROOT;
  }
  # Graphic Generation
  my $rtx_opt = "";
  if ($MATLAB || $ATU_GRAPHICS || $ENV{'ATURTX'})
  {
     $rtx_opt = " -g ";
  }

  if ($ATU_SPLIT_REPORT)
  {
    $rtx_opt = $rtx_opt . " -s" . $ATU_SPLIT_REPORT;
  }

  # Translator execution
  unlink ( "$atu_xrd" );

  my $headFile="$NAME.summary.txt";
  my $headOption;
  #The following line describe the report header content
  #lines are split on = sign to get 2 columns
  #line feed can be inserted using the 4 following chars: <br>
  #look for REPORTHEADER into the TDPeditor online help for more details
  if(open (HDR,">$headFile")){
     if($RTRT_ACTIVATED){ #=1 on eclipse
	print HDR "Project=$PROJECT_NAME\n" if($PROJECT_NAME ne '');
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_PATH\n" if($WORKSPACE_NAME ne '');
	print HDR "Working directory=$WORKING_DIR\n";
	print HDR "Test name=$NAME\n";
	print HDR "Report file=$atu_xrd\n";
	print HDR "\$PREPROCESS\n";
	print HDR "\$HEADERLINE\n";
     }else{
	print HDR "Project=$PROJECT_NAME\n" if($PROJECT_NAME ne '');
	print HDR "Project file=$PROJECT_PATH$PATH_SEPARATOR$PROJECT_NAME.rtp\n" if($PROJECT_NAME ne '');
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_NAME\n" if($WORKSPACE_NAME ne '');
	print HDR "Working directory=$WORKING_DIR\n";
	print HDR "Test node=$NAME\n";
	print HDR "Report file=".absolutePath($atu_xrd)."\n";
	print HDR "\$PREPROCESS\n"; # print the preprocessor command line in this place
	print HDR "\$HEADERLINE\n"; # print the test script header parameters in this place
     }
#	print HDR "=$\n";
     if(($FILEVERCMS) && ($FILEVERCMS ne '0'))
     {
        print &PerlTranslate("UnitTesting.pl", "Check file's version\n", "");
        print HDR "#Tested files version\n";
 	foreach my $f (@SOURCES) #(@INTEGRATEDFILENAMES,@ADDITONALFILES)
 	{
 		my $v=getFileVersion($f);
		$f=absolutePath($f);
 		print HDR "Tested File:$f=Version:$v\n";
	}
     }
     close HDR;
     $headOption=" \"-h$headFile\"";
  }

  if($Ini{'REPORTHEADER'}) {
     print "REPORTHEADER specified as:$LP_TARGETDIR/cmd/$Ini{'REPORTHEADER'}\n";
     print "sub reportHeader($headFile,$RTRT_ACTIVATED)\n";
     require "$TARGETDIR/cmd/$Ini{'REPORTHEADER'}";
     reportHeader($headFile,$RTRT_ACTIVATED);
  }

  my $atu_command = $BINPATH."rod2xrd" . $rtx_opt . $headOption. " \"-l$LOGFILENAME\"" . " \"-o$atu_xrd\" \"$atu_out.rod\"" ;
  my $trans_status = SystemP ($atu_command);
  $trans_status && die &PerlTranslate("UnitTesting.pl", "Xrd Translation Failed.\n");

  if(open (HDR,"$headFile")){
    if(open(LOG, ">>$LOGFILENAME")){
      while(<HDR>){
	if(/^Passed=(\d+)/){
	   print LOG "#OK $1\n";
	}
	if(/^Failed=(\d+)/){
	   if($1){
		   print LOG "#KO $1\n" ;
		   print "Test Failed $1\n";
	   }
	}
      } close LOG;
    } close HDR;
  }

  my ($reqXmld,$reqXml,$e)=SplitFileName($atu_xrd);$reqXml=$reqXmld.$reqXml.'.req.xml';
  my $req_command = $BINPATH."rod2req" . $headOption. " \"-l$LOGFILENAME\"" . " \"-o$reqXml\" \"$atu_out.rod\" -verbose" ;
  my $req_status = SystemP ($req_command);
  $req_status && print &PerlTranslate("UnitTesting.pl", "Requirement generation Failed.\n");
  copy($ENV{TESTRTDIR}.'/lib/reports/testreq_report.xsl',$reqXmld.'testreq_report.xsl'); #use $ENV for cmd line scripts (TestRT_test)

  Cat($headFile,"$NAME.atc.txt") if(($ATCMODE) && (-f "$NAME.atc.txt"));
  my $memSum=$ATP_TPF_NAME; $memSum=~s/\.tpf$/.atp.txt/;
  Cat($headFile,$memSum) if(($ATPMODE || $ATAMODE) && (-f $memSum));
  RegisterTmpFiles($headFile,$reqXmld.'testreq_report.xsl');

  if($ENV{RTRT_TEST_SUM}){
   Cat($ENV{RTRT_TEST_SUM},$headFile);
  }

  # Matlab Translation
  if ($MATLAB)
  {
     # MAT Generation
     my $atu_command = $BINPATH."rod2mat " . " \"$atu_out.rod\" \"$MATLAB_GENERATEDDATA\" \"$MATLAB_INFORMATIONFILE\"" ;
     my $trans_status = SystemP ($atu_command);
  }

  # add temporary files into log
  unlink("$atu_out.hdr");
  #$postpro_status == 3 when error in report
  return $postpro_status;
}

################################################################################
# Link, execute and postprocess for ATU node
sub UnitTesting
{
my $status = 0;
my $ada_mode = 0;
my $OUTFILE = $ATL_OUT_FILE;
my $codeChecker_status="x";

  # Out Files $RIOFILE is the var used by the Splitter
  if ($INTDIR)
  {
     $RIOFILE .= $INTDIR.$PATH_SEPARATOR;
  }
  $RIOFILE .= $ATU_INTERMEDIATE_FILE.".rio";

  if ($TDP_LANGUAGE=~/ADA/)
  {
     $ada_mode = 1;
  }

  if($MISRA_MODE && @XOB) {
   if ($STAGE =~ /e/ ) {
      $codeChecker_status=codeChecker_link() if($STAGE =~ /l/ );
   }else{
      $codeChecker_status=codeChecker_link(1); #request to check upToDate
   }
  }

#----------------------------------------------------------------------------
## Linkage
#----------------------------------------------------------------------------
 if ($STAGE =~ /l/)
 {
  if ($ada_mode)
  {
      my $cwdir = $WORKING_DIR;
      $cwdir=~s/\//$PATH_SEPARATOR/g;
      my $save_ctx = $ATL_MAIN_PROCEDURE;


      $ATL_MAIN_PROCEDURE = lc($ATU_INTERMEDIATE_FILE);
      $ATL_MAIN_PROCEDURE = clean_name_from_exotic_char($ATL_MAIN_PROCEDURE);
      $status = atl_link ($TESTPROGRAM_ABS, \@OBJECTS, $LDFLAGS,\@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);

      $ATL_MAIN_PROCEDURE = $save_ctx;
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
  }
  else
  {
      #----------------------------------------------------------------------------
      if ( $ATL_RUNTIME_FORMAT ne "NONE") {
         ## do not forget object file of target package.
         if (MakeTargetPackageObject()) {
  	   return 1;
         }
         push( @OBJECTS, GetTargetPackageLib($OBJEXT));
      }

      print &PerlTranslate("UnitTesting.pl", "Linking %1...\n", "%1 is the name of the program to be linked", $TESTPROGRAM);

      $status = atl_link ($TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                          \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
  }

  ## Do not forget the OBJ file in output log
  RegisterGenFiles("EXE $TESTPROGRAM");
  if ( $codeChecker_status eq ""){
    	print &PerlTranslate("UnitTesting.pl", "Code review report generation failed\n", "");
    	return 1;
  }
 }

#----------------------------------------------------------------------------
## Execution
#----------------------------------------------------------------------------
 if ($STAGE =~ /e/ )
 {

  if ($ada_mode)
  {
    unlink($RIOFILE); #keep old compatibility
    print &PerlTranslate("UnitTesting.pl", "Executing %1 %2 ...\n", "%1 is the program being executed,%2 is the args string", $TESTPROGRAM_ABS , $ARGUMENTS);
    $status = AtlExec ($TESTPROGRAM_ABS, $OUTFILE, $ARGUMENTS);
    RenameFile($RIOFILE,$OUTFILE) unless(-f $OUTFILE);  #keep old compatibility
  }
  else
  {
    $OUTFILE =~ s/\\/\\\\/;
    print &PerlTranslate("UnitTesting.pl", "Executing %1 %2 ...\n", "%1 is the program being executed,%2 is the args string", $TESTPROGRAM , $ARGUMENTS);
    $status = AtlExec ($TESTPROGRAM, $OUTFILE, $ARGUMENTS);
  }

  EndStep($status);
  if ($status != 0 )
  {
    print &PerlTranslate("UnitTesting.pl", "Executable terminated with error code %1.\n", "%1 is the value of the error code", $status);
  }
 }
 if ($STAGE =~ /[es]/ )
 {

  SplitTraces ($OUTFILE);

  unless (-f $RIOFILE )
  {
    print &PerlTranslate("UnitTesting.pl", "No trace %1 generated.\n", "%1 is the name of the trace file", $OUTFILE);
    open RIO,">".$RIOFILE; # create an empty file anyway
    close RIO;
  }
  if ( ! $used[$rio] )
  {
    print &PerlTranslate("UnitTesting.pl", "No RIO instruction found.\n");
#    return(1);
  }
 }

#----------------------------------------------------------------------------
## Postprocessing
#----------------------------------------------------------------------------
 $status =0;
 if ($STAGE =~ /r/ )
 {
  my $atu_cio = "$INTDIR".$PATH_SEPARATOR."atu.cio";
  print &PerlTranslate("UnitTesting.pl", "Postprocessing...\n");

  if (($ATCMODE) && (!$NO_COV_REPORT)) {
    if ( ! $used[$tio] )
    {
      print &PerlTranslate("UnitTesting.pl", "No TIO instruction found.\n");
      $ATC_REPORT_FAILED=1;
    }
    else
    {
      $status = atc_postpro ($ATC_TIO_NAME, $atu_cio);
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
    }
  }
  my $rio = $RIOFILE;
  $rio=~s/\.rio$//;
  
  coveragePost($TEST_NAME) if ( $ATCMODE );
  couplingPost($ATG_TGF_NAME,$rio.'.tdc') if( $ATGMODE );
  dataCouplingPost($DCP_DCP_NAME,$rio.'.tdc') if( $DCPMODE );
  stackSizePost($ATZ_TZF_NAME) if($ATAMODE && $ATZMODE);
  WCETPost($ATQ_TQF_NAME) if( $ATQMODE );#&& ($ATQ_COMPUTE_MINMAX==2));

  $status = atu_postpro ($rio, $XRDFILE, $atu_cio);
  die &PerlTranslate("UnitTesting.pl", "Failed.\n") if($status!=0 && $status!=3);
 }
 EndProcess();
return $status if($ECLIPSE_IDE);
return 0;
}

################################################################################
# Here is the beginning
################################################################################

unless($main::TestRTCC){

CheckIntDir();
CheckOutDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
exit (UnitTesting());
}

1;
#
################################################################################
