#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Unit Testing node -*-                            #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   @OBJECTS     : object files                                                #
#   $LDFLAGS     : link flags                                                  #
#   $LIBRARIES   : libraries list for linking                                  #
#   $TESTPROGRAM : executable name                                             #
#   $ARGUMENTS   : executable arguments                                        #
#   $EXECMODE    : execution mode ("NORMAL" or "DEBUG")                        #
#   $AMLFILE     : generated AML report                                        #
#   @OTSFILES    : list of OTS files                                           #
#   $BINDIR      : ATTOL binaries directory                                    #
#   $SCRIPTDIR   : Studio scripts directory                                    #
#   $TARGETDIR   : target package directory                                    #
#   $OUTDIR      : output directory                                            #
#   $LOGFILENAME : generated files log                                         #
################################################################################




# To do output in chronological order when stderr and stdout are both used
use FileHandle;
use File::Copy ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";

# Target package related functions
require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);
require $SCRIPTDIR . "CmsInfo.pl";

# Target-dependant functions
require $TARGETDIR ."/cmd/atl_func.pl";
require $TARGETDIR ."/cmd/atl_link.pl";
require $TARGETDIR ."/cmd/atl_exec.pl";

#$NO_JUSTIFY=1 if($ATVMODE && !(defined $NO_JUSTIFY));
################################################################################
# Link, execute and postprocess for ATV node
sub VisualTesting
{
my $status = 0;
my $OUTFILE = $ATL_OUT_FILE;
my $codeChecker_status="x";

  # Out Files $RIOFILE is the var used by the Splitter
  $RIOFILE = $OUTFILE; $RIOFILE=~ s,\.spt$,.rio,;


  if($MISRA_MODE && @XOB) {
   if ($STAGE =~ /e/ ) {
      $codeChecker_status=codeChecker_link() if($STAGE =~ /l/ );
   }else{
      $codeChecker_status=codeChecker_link(1); #request to check upToDate
   }
  }

#----------------------------------------------------------------------------
## Linkage
#----------------------------------------------------------------------------
 if ($STAGE =~ /l/)
 {
#  if ($ada_mode)
#  {
#      my $cwdir = $WORKING_DIR;
#      $cwdir=~s/\//$PATH_SEPARATOR/g;
#      my $save_ctx = $ATL_MAIN_PROCEDURE;
#
#
#      $ATL_MAIN_PROCEDURE = lc($ATU_INTERMEDIATE_FILE);
#      $ATL_MAIN_PROCEDURE = clean_name_from_exotic_char($ATL_MAIN_PROCEDURE);
#      $status = atl_link ($TESTPROGRAM_ABS, \@OBJECTS, $LDFLAGS,\@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
#
#      $ATL_MAIN_PROCEDURE = $save_ctx;
#      $status && die &PerlTranslate("VisualTesting.pl", "Failed.\n");
#  }
#  else
  {
      #----------------------------------------------------------------------------
      if ( $ATL_RUNTIME_FORMAT ne "NONE") {
         ## do not forget object file of target package.

         print "TP...\n";
         if (MakeTargetPackageObject()) {
  	   return 1;
         }
         push( @OBJECTS, GetTargetPackageLib($OBJEXT));
      }

      print &PerlTranslate("VisualTesting.pl", "Linking %1...\n", "%1 is the name of the program to be linked", $TESTPROGRAM);

      $status = atl_link ($TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                          \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
      EndStep($status);
      $status && die &PerlTranslate("VisualTesting.pl", "Failed.\n");
  }

  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("VisualTesting.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "EXE $TESTPROGRAM\n";
  if(($FILEVERCMS) && ($FILEVERCMS ne '0'))
  {
        print &PerlTranslate("UnitTesting.pl", "Check file's version\n", "");
 	foreach my $f (@SRC_UNDERTEST)
 	{
 		my $v=getFileVersion($f);
		#$f=absolutePath($f);
 		print LOG "SVS \"$f\" \"$v\"\n";
	}
  }
  close( LOG );
  if ( $codeChecker_status eq ""){
    	print &PerlTranslate("VisualTesting.pl", "Code review report generation failed\n", "");
    	return 1;
  }
 }

#----------------------------------------------------------------------------
## Execution
#----------------------------------------------------------------------------
 if ($STAGE =~ /e/ )
 {

#  if ($ada_mode)
#  {
#    unlink($RIOFILE); #keep old compatibility
#    print &PerlTranslate("VisualTesting.pl", "Executing %1...\n", "%1 is the program being executed", $TESTPROGRAM_ABS . $ARGUMENTS);
#    $status = AtlExec ($TESTPROGRAM_ABS, $OUTFILE, $ARGUMENTS);
#    RenameFile($RIOFILE,$OUTFILE) unless(-f $OUTFILE);  #keep old compatibility
#  }
#  else
  {
    $OUTFILE =~ s/\\/\\\\/;
    print &PerlTranslate("VisualTesting.pl", "Executing %1 %2 ...\n", "%1 is the program being executed,%2 the args string", $TESTPROGRAM , $ARGUMENTS);
    $status = AtlExec ($TESTPROGRAM, $OUTFILE, $ARGUMENTS);
  }

  EndStep($status);
 }
 if ( $STAGE =~ /[es]/ )
 {
  if ($status != 0 )
  {
    print &PerlTranslate("VisualTesting.pl", "Executable terminated with error code %1.\n", "%1 is the value of the error code", $status);
  }

  SplitTraces ($OUTFILE);

  unless (-f $RIOFILE )
  {
    print &PerlTranslate("VisualTesting.pl", "No trace %1 generated.\n", "%1 is the name of the trace file", $OUTFILE);
    open RIO,">".$RIOFILE; # create an empty file anyway
    close RIO;
  }
  if ( ! $used[$rio] )
  {
    print &PerlTranslate("VisualTesting.pl", "No RIO instruction found.\n");
#    return(1);
  }
 }

#----------------------------------------------------------------------------
## Postprocessing
#----------------------------------------------------------------------------
 if ($STAGE =~ /r/ )
 {
  my $atu_cio = "$INTDIR".$PATH_SEPARATOR."atu.cio";
  print &PerlTranslate("VisualTesting.pl", "Postprocessing...\n");
  #place to generate covJustify if required at test level
  unless($NO_JUSTIFY) {
    $ATC_JUSTIFY= genJustifyFDC(\@FDCFILES);
    RegisterGenFiles("TIO $ATC_JUSTIFY") if($ATC_JUSTIFY ne '');
  }
  coveragePost($TEST_NAME) if ( $ATCMODE );
  couplingPost($ATG_TGF_NAME) if( $ATGMODE );
  dataCouplingPost($DCP_DCP_NAME) if( $DCPMODE );
  stackSizePost($ATZ_TZF_NAME) if($ATAMODE && $ATZMODE);
  WCETPost($ATQ_TQF_NAME) if( $ATQMODE );#&& ($ATQ_COMPUTE_MINMAX==2));
  
  if (($ATCMODE) && (!$NO_COV_REPORT)) {
    if ( ! $used[$tio] )
    {
      print &PerlTranslate("VisualTesting.pl", "No TIO instruction found.\n");
      $ATC_REPORT_FAILED=1;
    }
    else
    {
      my $atc_conclusion="$NAME.atc.txt";
      $status = atc_postpro ($ATC_TIO_NAME, $atu_cio);
      unless($status||$ATC_REPORT_FAILED){
      #print "\n$atu_cio\n";
      	if(open (CIO, $atu_cio)){
      	  my $testIdx=-1;
      	  my   @zero=();
      	  my   @covFunc=();
      	  my   @covFuncTotal=();
      	  my   @covfuncPlus=();
      	  my   @covCalls=();
      	  my   @covCallsTotal=();
      	  my   @covCallsPlus=();
      	  my   @covBlock=();
      	  my   @covBlockTotal=();
      	  my   @covBlockPlus=();
      	  my   @covImp=();
      	  my   @covImpTotal=();
      	  my   @covImpPlus=();
      	  my   @covDecis=();
      	  my   @covDecisTotal=();
      	  my   @covDecisPlus=();
      	  my   @covLoops=();
      	  my   @covLoopsTotal=();
      	  my   @covLoopsPlus=();
      	  my   @covBasics=();
      	  my   @covBasicsTotal=();
      	  my   @covBasicsPlus=();
      	  my   @covModified=();
      	  my   @covModifiedTotal=();
      	  my   @covModifiedPlus=();
      	  my $inTest=0;
      	  my $tcNum=0;
      	  my $buffer='';
      	  while(<CIO>){
		  #we keep only O0 block all other Ox are ignored
      	  	if(/^O(\d)/) {
      	  	   $inTest=0;
      	  	   $inTest=1 if($1 == 0);
      	  	   next;
      	  	}
      	  	if(($inTest==1) && (/^T(\d+)/)){
      	  	   $buffer='';
      	  	   $testIdx=-1;
      	  	   #print ">>>>>>>>>$covBlockTotal{0}\n";
      	  	   $inTest=2 ;
      	  	}
      	  	if($inTest==2){
      	  	   if(/^T(\d+)/){
		      my $prev=$testNum{$testIdx};
      	  	      $testIdx++;
      	  	      $testNum{$testIdx}=$1;
      	  	      $tcNum++ if($prev != $1);
			$_="TestCase $tcNum covers\n";
      	  	      $covFunc{$testIdx}=0;
      	  	      $covFuncTotal{$testIdx}=0;
      	  	      $covfuncPlus{$testIdx}=0;
        	      $covCalls{$testIdx}=0;
      	  	      $covCallsTotal{$testIdx}=0;
      	  	      $covCallsPlus{$testIdx}=0;
       	  	      $covBlock{$testIdx}=0;
      	  	      $covBlockTotal{$testIdx}=0;
      	  	      $covBlockPlus{$testIdx}=0;
      	  	      $covImp{$testIdx}=0;
      	  	      $covImpTotal{$testIdx}=0;
      	  	      $covImpPlus{$testIdx}=0;
      	  	      $covDecis{$testIdx}=0;
      	  	      $covDecisTotal{$testIdx}=0;
      	  	      $covDecisPlus{$testIdx}=0;
      	  	      $covLoops{$testIdx}=0;
      	  	      $covLoopsTotal{$testIdx}=0;
      	  	      $covLoopsPlus{$testIdx}=0;
      	  	      $covBasics{$testIdx}=0;
      	  	      $covBasicsTotal{$testIdx}=0;
      	  	      $covBasicsPlus{$testIdx}=0;
      	  	      $covModified{$testIdx}=0;
      	  	      $covModifiedTotal{$testIdx}=0;
      	  	      $covModifiedPlus{$testIdx}=0;
   	  	   }
      	  	   elsif(/Functions( and exits)?\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covFunc{$testIdx}+=$2;
      	  	      $covFuncTotal{$testIdx}+=$3;
      	  	      $covfuncPlus{$testIdx}+=$4;
      	  	   }
      	  	   elsif(/Calls\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covCalls{$testIdx}+=$1;
      	  	      $covCallsTotal{$testIdx}+=$2;
      	  	      $covCallsPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Statement blocks\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covBlock{$testIdx}+=$1;
      	  	      $covBlockTotal{$testIdx}+=$2;
      	  	      $covBlockPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Implicit blocks\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covImp{$testIdx}+=$1;
      	  	      $covImpTotal{$testIdx}+=$2;
      	  	      $covImpPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Decisions\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covDecis{$testIdx}+=$1;
      	  	      $covDecisTotal{$testIdx}+=$2;
      	  	      $covDecisPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Loops\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covLoops{$testIdx}+=$1;
      	  	      $covLoopsTotal{$testIdx}+=$2;
      	  	      $covLoopsPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Basic conditions\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covBasics{$testIdx}+=$1;
      	  	      $covBasicsTotal{$testIdx}+=$2;
      	  	      $covBasicsPlus{$testIdx}+=$3;
      	  	   }
      	  	   elsif(/Modified conditions\s+.*\s+\((\d+)\/(\d+)\).*\s+\(\+(\d+)\)/){
      	  	      $covModified{$testIdx}+=$1;
      	  	      $covModifiedTotal{$testIdx}+=$2;
      	  	      $covModifiedPlus{$testIdx}+=$3;
      	  	   }
      	  	   $buffer.=$_;
      	  	}
      	  }
      	  close CIO;
      	  print $buffer;
      	  if(open(RIO,">>$RIOFILE"))
      	  {
		  my $idx=0;
		  my @Iter;
		  while($idx <= $testIdx){#P=FunctionAndExits B=block R=>D=Decision  L=Loop I=Implicit C=BasicConditions
		     $Iter[$testNum{$idx}]++;
		     print RIO "IP $testNum{$idx} $Iter[$testNum{$idx}] $covFunc{$idx} $covFuncTotal{$idx} $covFuncPlus{$idx}\n" if($covFuncTotal{$idx});
		     print RIO "IB $testNum{$idx} $Iter[$testNum{$idx}] $covBlock{$idx} $covBlockTotal{$idx} $covBlockPlus{$idx}\n" if($covBlockTotal{$idx});
		     print RIO "II $testNum{$idx} $Iter[$testNum{$idx}] $covImp{$idx} $covImpTotal{$idx} $covImpPlus{$idx}\n" if($covImpTotal{$idx});
		     print RIO "IL $testNum{$idx} $Iter[$testNum{$idx}] $covLoops{$idx} $covLoopsTotal{$idx} $covLoopsPlus{$idx}\n" if($covLoopsTotal{$idx});
		     print RIO "IA $testNum{$idx} $Iter[$testNum{$idx}] $covCalls{$idx} $covCallsTotal{$idx} $covCallsPlus{$idx}\n" if($covCallsTotal{$idx});
		     print RIO "ID $testNum{$idx} $Iter[$testNum{$idx}] $covDecis{$idx} $covDecisTotal{$idx} $covDecisPlus{$idx}\n" if($covDecisTotal{$idx});
		     print RIO "IC $testNum{$idx} $Iter[$testNum{$idx}] $covBasics{$idx} $covBasicsTotal{$idx} $covBasicsPlus{$idx}\n" if($covBasicsTotal{$idx});
		     print RIO "IM $testNum{$idx} $Iter[$testNum{$idx}] $covModified{$idx} $covModifiedTotal{$idx} $covModifiedPlus{$idx}\n" if($covModifiedTotal{$idx});
		     $idx++;
		  }
		  #get global coverage
		  my $gcovFunc="";
		  if(open (CIO, $atc_conclusion)){
		     while(<CIO>){
				if(/Functions\s+.*\s+\((\d+)\/(\d+)\)/){ $gcovFunc= "IP -1 $1 $2\n";}
				else {print RIO $gcovFunc ; $gcovFunc=""; }
				if(/Statement blocks\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"IB -1 $1 $2\n";}
				elsif(/Calls\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"IA -1 $1 $2\n";}
				elsif(/Implicit blocks\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"II -1 $1 $2\n";}
				elsif(/Decisions\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"ID -1 $1 $2\n";}
				elsif(/Loops\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"IL -1 $1 $2\n";}
				elsif(/Basic conditions\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"IC -1 $1 $2\n";}
				elsif(/Modified conditions\s+.*\s+\((\d+)\/(\d+)\)/){ print RIO $gcovFunc,"IM -1 $1 $2\n";}
		     }
		     close CIO;
		  }
	     close RIO;
	  }
      	}
      }
    }
  }
#post processing place if any
 }
 print &PerlTranslate("VisualTesting.pl", "Done %1\n","",GetDate());
 return 0;
}

sub TestDriverCompil
{
  my $src = shift;
  my ($srcdir,$srcBaseName,$e)=SplitFileName($src);
  my $obj = $srcBaseName.'.'.$OBJEXT;
#     $obj=~s,$SRCEXT$,$OBJEXT,;
  my $status=0;

   if (($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP"))
   {
     unless(-f $src){
          print STDERR &PerlTranslate("VisualTesting.pl", "file: %1 not found. please save your test harness then rebuild\n", "%1 is the harness source", $src);
     	  return 1;
     }
     if(($TDP_LANGUAGE eq "C") && (grep(/^RTRT_CPP$/,@GENERATION_CONDITION))) {
		my $nextTDPname=$Ini{NAME};$nextTDPname=~s,C\s,C++ ,;
          	print STDERR &PerlTranslate("VisualTesting.pl", "\nERROR: It is preferable to use the C++ TDP for C++ project\nchange the project type or change project properties > C/C++ build > settings > build TDP > 'Target Deployment Port' to %1\n","%1 is the TDP name",$nextTDPname);
          	return 1;  
			#print STDERR "remove ",pop(@GENERATION_CONDITION), "\n";#for debug only
     }
	 RegisterGenFiles("TSF $srcdir$srcBaseName.tsf") if(-f "$srcdir$srcBaseName.tsf");
	 $TDCFILE="$srcdir$srcBaseName.tdc" if(-f "$srcdir$srcBaseName.tdc");
     if ((((@SRC_UNDERTEST) && !$IS_SEPARATE) && ($SRC_UNDERTEST[0] ne $EXCLUDE_INSTR_SRC[0])) || ($EXECMODE eq 'DEBUG'))
     {
	   ## Do not forget the OBJ file in output log
	     my $atu_has_rename = 0;
	     my $atu_rename_file="";
	     if(@STUBLIST)
	     {
	       s!,!\\,!g foreach(@STUBLIST);
	       $atu_rename_file="stubs_rename.opt";
	       if(open (REN, ">$atu_rename_file"))
	       {
		  my $action='RENAME'; $action='REMOVE' if($TDP_LANGUAGE eq "CPP");
		  $atu_has_rename =1;
		  print REN "-$action=\"".join("\",\\\n\"",@STUBLIST)."\"\n";
		  close REN;
	       }
	       else {
		  print &PerlTranslate("VisualTesting.pl", "Cannot open rename file: %1\n", "%1 is the reason given by the Operating System", $!);
	       }
	     }
	     if ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || ($ATAMODE && $ATZMODE) || $ATGMODE || $DCPMODE || $atu_has_rename || ($EXECMODE eq 'DEBUG')) # force instrum in debug to change #line
	     {
	      ## Tester instrumentation
	      ## Intermediate files
	      my $prepro_file = $src;
		 $prepro_file=~s,$SRCEXT$,i,;
	      my $instr_file = $prepro_file;
		 $instr_file=~s,\.i$,_aug.$SRCEXT,;
#		push (@COMPLETE_INCLUDE_DIR_LIST,"$INTDIR");
#		push (@COMPLETE_INCLUDE_DIR_LIST,"$TARGETDIR" . "$PATH_SEPARATOR" . "lib");

	      ## C preprocessing
	      print &PerlTranslate("VisualTesting.pl", "%1: Preprocessing...\n", "%1 is the file to be preprocessed", $src);

	      $status = atl_cpp ($TDP_LANGUAGE, $src, $prepro_file, $CPPFLAGS.' '.$PPFLAGS,
				 \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST);
	      EndStep($status);
	      return($status) if($status);

	     if ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || ($ATAMODE && $ATZMODE) || $ATGMODE || $DCPMODE || $atu_has_rename)
	     {
	      # Class ATTOL and $src are not instrumented
	      my $instr_opts = BuildCCppInstrOptions($TDP_LANGUAGE,'',
						    ($TDP_LANGUAGE eq "CPP") ? "ATTOL" : "",
						       $atu_rename_file);

	      #if(@SRC_UNDERTEST) {
		   $instr_opts .=" -FILE=\"$SRC_UNDERTEST[0]\"";
	      #} else {
	      #	   print &PerlTranslate("VisualTesting.pl", "WARNING: No file under test defined\n", "");
	      #}
	      my ($srcdir, $srcname, $srcext) = SplitFileName( $src );
	      $status = atl_instr ($TDP_LANGUAGE, $prepro_file, $instr_file, $instr_opts,$srcname.'.'.$srcext);
	      rename ($atu_rename_file,$INTDIR.'/'.$atu_rename_file.'.bak') if($atu_rename_file ne '');

	      EndStep($status);
	      return($status) if($status);
	     } else {
	      rename($prepro_file,$instr_file);
	     }
	      removeInternalRefLine($instr_file) if($EXECMODE eq 'DEBUG');
	      $src=$instr_file;
	     }
     }
     ## Compilation
     my @conditions=@GENERATION_CONDITION;
     push(@conditions,"RTRT_TEST_AS_SEPARATE") if($IS_SEPARATE);
     print &PerlTranslate("VisualTesting.pl", "%1: Compiling...\n", "%1 is file to be compiled", $src);
     $status = atl_cc ($TDP_LANGUAGE, $src, $obj, $CPPFLAGS . " " . $CFLAGS,
				\@conditions, \@COMPLETE_INCLUDE_DIR_LIST);

     if($status){
	      print &PerlTranslate("VisualTesting.pl", "%1: Compilation failed, generate Preprocessed file %2 for analyse\n", "%1 is the source file, %2 is the generated file", $src,"$obj.i");
		atl_cpp ($TDP_LANGUAGE, $src, "$obj.i", $CPPFLAGS.' '.$PPFLAGS,
				 \@conditions, \@COMPLETE_INCLUDE_DIR_LIST);
     }
     EndStep($status);
     if(open ( LOG4, ">>$LOGFILENAME" ))
     {
	if($instr_file ne '')
	{
           print  LOG4 "DUM $instr_file\n";
           print  LOG4 "DUM $prepro_file\n";
        }
       print LOG4 "OBJ $obj\n" unless($status);
       close( LOG4 );
     }
     else {
        print &PerlTranslate("VisualTesting.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
     }
   }
   return $status;
}

sub removeInternalRefLine
{
  my $fileIn = shift;
  my $lPROJECT_PATH=$PROJECT_PATH.'/';$lPROJECT_PATH=~s,\\,/,g;$lPROJECT_PATH=~s,\/\/,/,g;
  if(open(OUT_FILE, ">$fileIn.tmp")) {
	if(open(THIS_FILE, "<$fileIn")) {
		while(<THIS_FILE>) {
		    s,^\s*\#(line)?\s1\s\".*$, , if(/\|/);
		    s,\"\/$PROJECT_NAME\/,\"$lPROJECT_PATH, if(/^\s*\#(line)?\s\d/);
		    #s,^\#line\s\d+\s\"\/$PROJECT_NAME\/.*$, ,;
		    print OUT_FILE;
		}
		close(THIS_FILE);
		close(OUT_FILE);
		rename("$fileIn.tmp",$fileIn);
	} else {
		print &PerlTranslate("VisualTesting.pl", "Cannot open %1\n", "%1 is the file that cannot be opened", $fileIn);
		close(OUT_FILE);
	}
  } else {
	print &PerlTranslate("VisualTesting.pl", "Cannot write file %1.tmp\n", "%1 is the file that cannot be opened", $fileIn);
  }
}
1;
#
################################################################################

